/****************************************************************************
**
** Copyright (C) 2016 Openismus GmbH.
** Author: Peter Penz (ppenz@openismus.com)
** Author: Patricia Santana Cruz (patriciasantanacruz@gmail.com)
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "makestep.h"
#include "autotoolsprojectconstants.h"

#include <projectexplorer/buildsteplist.h>
#include <projectexplorer/projectexplorerconstants.h>

using namespace AutotoolsProjectManager::Constants;

namespace AutotoolsProjectManager {
namespace Internal {

// MakeStep

class MakeStep : public ProjectExplorer::MakeStep
{
public:
    MakeStep(ProjectExplorer::BuildStepList *bsl, Utils::Id id);
};

MakeStep::MakeStep(ProjectExplorer::BuildStepList *bsl, Utils::Id id)
    : ProjectExplorer::MakeStep(bsl, id)
{
    setAvailableBuildTargets({"all", "clean"});
    if (bsl->id() == ProjectExplorer::Constants::BUILDSTEPS_CLEAN) {
        setSelectedBuildTarget("clean");
        setIgnoreReturnValue(true);
    } else {
        setSelectedBuildTarget("all");
    }
}

// MakeStepFactory

MakeStepFactory::MakeStepFactory()
{
    registerStep<MakeStep>(MAKE_STEP_ID);
    setDisplayName(ProjectExplorer::MakeStep::defaultDisplayName());
    setSupportedProjectType(AUTOTOOLS_PROJECT_ID);
}

} // Internal
} // AutotoolsProjectManager
