/* File generated by Qt Creator */

import QmlProject 1.1

Project {
    mainFile: "%{MainQmlFile}"
    qtForMCUs: true

    /* Include .qml, .js, and image files from current directory and subdirectories */
    QmlFiles {
        directory: "."
    }
    JavaScriptFiles {
        directory: "."
    }
    ImageFiles {
        directory: "."
    }
}
