using Fake;

public class HelloWorld
{
    public static void Main()
    {
        System.Console.WriteLine("Hello world!");
        System.Console.WriteLine(Mod.Cool());
        System.Console.WriteLine(Libby.Cool());
        System.Console.WriteLine(Libby2.Cool());
    }
}
