/****************************************************************************
**
** Copyright (C) 2020 Denis Shienkov <denis.shienkov@gmail.com>
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include <QXmlStreamAttribute>

namespace BareMetal {
namespace Gen {
namespace Xml {

class Project;
class ProjectOptions;
class Property;
class PropertyGroup;

class INodeVisitor
{
public:
    virtual ~INodeVisitor() {}

    virtual void visitProjectOptionsStart(const ProjectOptions *projectOptions)
    { Q_UNUSED(projectOptions) }
    virtual void visitProjectOptionsEnd(const ProjectOptions *projectOptions)
    { Q_UNUSED(projectOptions) }

    virtual void visitProjectStart(const Project *project) { Q_UNUSED(project) }
    virtual void visitProjectEnd(const Project *project) { Q_UNUSED(project) }

    virtual void visitPropertyStart(const Property *property) = 0;
    virtual void visitPropertyEnd(const Property *property) = 0;

    virtual void visitPropertyGroupStart(const PropertyGroup *propertyGroup) = 0;
    virtual void visitPropertyGroupEnd(const PropertyGroup *propertyGroup) = 0;
};

} // namespace Xml
} // namespace Gen
} // namespace BareMetal
